/*****************************************************************************
 Copyright (c) 1999-2000, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

****************************************************************************/

#include "hamcore.h"
#include <linux/sched.h>
#include <asm/system.h>
#include <asm/irq.h>

#include "tasker.h"

extern ubyte acu_loop;
extern unsigned int* dsp_irq;

void disable_processor_interrupts(void);
void enable_processor_interrupts(void);

void fatal_error(byte *file, int line);
void fault_handler(ubyte);

/* global variables */
ubyte sys_intr_disabled = FALSE;
uword num_fatal_errors = 0;

ubyte fatal_error_occurred = FALSE;
ubyte fatal_msg_displayed = FALSE;


void modem_disable_interrupts(void)
{
   /* stubbed */
}

void modem_enable_interrupts(void)
{
   /* stubbed */
}

void line_enable_interrupts(void)
{
   enable_irq(*dsp_irq);
}

void line_disable_interrupts(void)
{
   disable_irq(*dsp_irq);
}

void disable_processor_interrupts(void)
{
   line_disable_interrupts();
   sys_intr_disabled = TRUE;
}

void enable_processor_interrupts(void)
{
   line_enable_interrupts();
   sys_intr_disabled = FALSE;
}

void disable_int(void)
{
  sys_intr_disabled = TRUE;
  disable_irq(*dsp_irq);
}

void enable_int(void)
{
  sys_intr_disabled = FALSE;
  enable_irq(*dsp_irq);
}

/*******************************************************************************
/ Function:  CLM_ALERT
/
/ Remarks:   This routine processes ALERT events.
/
/ Inputs:    none
/
/ Outputs:   none
*******************************************************************************/
void CLM_ALERT(void)
{
   Debug_Printf("CLM_ALERT: unexpected event\n");

   while (1)
   {
      RSATaskYield();
   }
}

/*******************************************************************************
/ Function:  InitFatalErrorHandler
/
/ Remarks:   This routine initialize the fatal error handler variables.
/
/ Inputs:    none
/
/ Outputs:   none
*******************************************************************************/
void InitFatalErrorHandler(void)
{
   num_fatal_errors = 0;
   fatal_error_occurred = FALSE;
   fatal_msg_displayed = FALSE;
}

/*******************************************************************************
/ Function:  fatal_error
/
/ Remarks:   This routine performs fatal error handling.
/
/ Inputs:    file - file where fatal error occurred
/            line - line number where fatal error occurred
/
/ Outputs:   none
*******************************************************************************/
void fatal_error(byte *file, int line)
{
   /* Only display the warning message once*/
   if (!fatal_msg_displayed)
   {
      Debug_Printf("CLM FATAL ERROR!  File: %s at Line: %d\n", file, line);
      fatal_msg_displayed = TRUE;
   }

   /* try to recover fatal errors in release builds */
   #ifdef RELEASE_BUILD
      fault_handler(0);
   #endif
}

/*******************************************************************************
/ Function:  fault_handler
/
/ Remarks:   This routine processes system faults.
/
/ Inputs:    code
/
/ Outputs:   none
*******************************************************************************/
void fault_handler(ubyte code)
{
   fatal_error_occurred = TRUE;
   acu_loop = FALSE;

   Debug_Printf("fault_handler: code=%d\n", code);

   while (1)
   {
      RSATaskYield();
   }
}
